using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixStatistics.Class;
using ZedGraph;
using Utils.Misc;
using PhoenixDLL.GUI;
using Utils.DataBase;
using PhoenixDLL.Class;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Managers;

namespace PhoenixStatistics.GUI
{
    /// <summary>
    /// Fentre principale pour les statistiques
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Manager for the statistics
        /// </summary>
        private StatManager vStatManager = new StatManager();
        /// <summary>
        /// Sql manager for queries
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();

            #region Corrector
            nud_Age.Value = 100;
            #endregion
        }
        #endregion

        #region Event
        /// <summary>
        /// Chargement de la page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void fr_Main_Load(object sender, EventArgs e)
        {
            InitializeGraphMeetingPerSeason();
            InitializeGraphAges();
            InitializeGraphSex();
            InitializeAdditionalAge();
        }
        /// <summary>
        /// Clique sur A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_About(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Click on search patients older than
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_PatientOlderThan(object sender, EventArgs e)
        {
            RefreshOlderThan();
        }
        /// <summary>
        /// ContextMenuStrip modify this old patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModifyOlder(object sender, EventArgs e)
        {
            int IDPatient = (int)dgv_Age.SelectedRows[0].Cells["ID_Patient"].Value;
            PatientManager pm = PatientManager.Create(IDPatient);
            fr_PatientManager fr = new fr_PatientManager();
            fr.Manager = pm;
            fr.Use = Use.Update;
            fr.Text = "Modification d'un patient";
            fr.Size = new Size(fr.Size.Width, 600);
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
            {
                RefreshOlderThan();
                if (InstanceInterData.Instance.Stata_AskReboot)
                {
                    dr = MessageBox.Show("Vous avez modifi un patient. Les graphiques ne sont plus  jour. "
                        + "Voulez-vous redmarrer l'application?", "Question", MessageBoxButtons.YesNo);
                    if (dr == DialogResult.No && InstanceInterData.Instance.Stata_AskAskReboot)
                    {
                        MessageBox.Show("Voulez vous que l'on demande  nouveau si il faut redmarre "
                        + "l'application?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (dr == DialogResult.No)
                            InstanceInterData.Instance.Stata_AskReboot = false;
                    }
                    else
                    {
                        Application.Restart();
                    }
                }
            }
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Calcule les statistiques
        /// </summary>
        public void CalculateStatistice()
        {
            vStatManager.CalculateMeetingOnMonth();
            vStatManager.CalculateAges();
            vStatManager.CalculateWomenAges();
            vStatManager.CalculateMenAges();
            vStatManager.CalculateAllCount();
            vStatManager.CalculateAllAverageAges();
        }
        /// <summary>
        /// Initialise la graphe des rendez-vous par saisons
        /// </summary>
        private void InitializeGraphMeetingPerSeason()
        {
            GraphPane Panel = zgc_RdvOnMonth.GraphPane;
            Panel.Title.Text = "Rendez-vous par mois";
            Panel.XAxis.Title.Text = "Mois";
            Panel.XAxis.Type = AxisType.Text;
            Panel.YAxis.Title.Text = "Nombre de rendez-vous";
            Panel.CurveList.Clear();

            BarItem[] BarItems = new BarItem[12];
            Color[] color = {Color.SkyBlue, Color.SlateBlue, Color.SlateGray, Color.YellowGreen, Color.SpringGreen, Color.SteelBlue,
                    Color.Tan, Color.Teal, Color.Thistle, Color.Tomato, Color.Transparent, Color.Turquoise};

            for (int i = 0; i < 12; i++)
            {
                double[] buf = { vStatManager.MeetingOnMonth[i] };
                BarItems[i] = Panel.AddBar(FrenchMonth.GetNameOf(i + 1), null, buf, Color.Red);
                BarItems[i].Bar.Fill = new Fill(color[i], Color.White, color[i]);
            }


            zgc_RdvOnMonth.Invalidate();
            zgc_RdvOnMonth.AxisChange();
        }
        /// <summary>
        /// Initialise la graphe des ages des patients
        /// </summary>
        private void InitializeGraphAges()
        {
            GraphPane TotalPanel = zgc_Ages.GraphPane;
            TotalPanel.Title.Text = "Graphique des ages totaux.";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            TotalPanel.CurveList.Clear();

            LineItem Courbe = TotalPanel.AddCurve("Age des patients", vStatManager.AgeList, vStatManager.Ages,
                Color.Orange, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            Courbe = TotalPanel.AddCurve("Age des femmes", vStatManager.AgeList, vStatManager.WomenAges,
                Color.DeepPink, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            Courbe = TotalPanel.AddCurve("Age des hommes", vStatManager.AgeList, vStatManager.MenAges,
                Color.Navy, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            zgc_Ages.AxisChange();
            zgc_Ages.Invalidate();

        }
        /// <summary>
        /// Initialise le graph des sexe des patients
        /// </summary>
        private void InitializeGraphSex()
        {
            GraphPane TotalPanel = zgc_Sex.GraphPane;
            TotalPanel.Title.Text = "Graphique des sexes.";
            TotalPanel.CurveList.Clear();

            if (vStatManager.CountAll != 0)
            {
                GraphPane Panel = zgc_Sex.GraphPane;
                float pctHomme = ((float)vStatManager.CountMen / (float)vStatManager.CountAll) * 100;
                float pctFemme = (float)((float)vStatManager.CountWomen / (float)vStatManager.CountAll) * 100;

                string title = vStatManager.CountAll + " patients (" + vStatManager.CountMen
                    + " homme et " + vStatManager.CountWomen + " femmes).";

                PieItem PieMen = Panel.AddPieSlice(pctHomme, Color.Blue, Color.White, 45F, 0.1, "Hommes ( "
                    + vStatManager.CountMen + " - " + pctHomme.ToString("0.00") + "%)");
                PieItem PieWomen = Panel.AddPieSlice(pctFemme, Color.Pink, Color.White, 45F, 0.1, "Femmes ( "
                    + vStatManager.CountWomen + " - " + pctFemme.ToString("0.00") + "%)");

                // Additions des parts de tartes    
                CurveList curves = Panel.CurveList;
                double Total = 0;
                for (int i = 0; i < curves.Count; i++)
                    Total += ((PieItem)curves[i]).Value;

                zgc_Sex.AxisChange();
                zgc_Sex.Invalidate();
            }

        }
        /// <summary>
        /// Initialise les graphs additionnels
        /// </summary>
        private void InitializeAdditionalAge()
        {
            lbl_AverageAge.Text = "Age moyen des patients: " + vStatManager.AverageAge.ToString("0.00") + " ans";
            lbl_MaxAgeFemme.Text = "Age moyen des femmes: " + vStatManager.AverageWomenAge.ToString("0.00") + " ans";
            lbl_MaxAgeHomme.Text = "Age moyen des hommes: " + vStatManager.AverageMenAge.ToString("0.00") + " ans";


            GraphPane TotalPanel = zgc_AgeMoyen.GraphPane;
            TotalPanel.Title.Text = "Age des patients";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            LineItem Courbe = TotalPanel.AddCurve("Age des patients", vStatManager.AgeList, vStatManager.Ages,
                Color.Orange, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            TotalPanel = zgc_AgeMoyenFemmes.GraphPane;
            TotalPanel.Title.Text = "Age des femmes";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            Courbe = TotalPanel.AddCurve("Age des femmes", vStatManager.AgeList, vStatManager.WomenAges,
                Color.DeepPink, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            TotalPanel = zgc_AgeMoyenHommes.GraphPane;
            TotalPanel.Title.Text = "Age des hommes";
            TotalPanel.XAxis.Title.Text = "Age";
            TotalPanel.YAxis.Title.Text = "Nombre de patients";
            Courbe = TotalPanel.AddCurve("Age des hommes", vStatManager.AgeList, vStatManager.MenAges,
                Color.Navy, SymbolType.None);
            Courbe.Line.Width = 2.0F;

            zgc_AgeMoyen.AxisChange();
            zgc_AgeMoyenFemmes.AxisChange();
            zgc_AgeMoyenHommes.AxisChange();

            zgc_AgeMoyen.Invalidate();
            zgc_AgeMoyenFemmes.Invalidate();
            zgc_AgeMoyenHommes.Invalidate();
        }
        /// <summary>
        /// Fill the DataGridView of the patient older than.
        /// </summary>
        private void RefreshOlderThan()
        {
            #region SQL Query
            dgv_Age.DataSource = null;
            DateTime dt = new DateTime((int)(DateTime.Today.Year - nud_Age.Value), 1, 1);
            string SQLQuery = "SELECT * FROM Patient WHERE DateNaissance < FORMAT(\"" + dt.ToString("d")
                + "\",\"dd/mm/yyyy\")";
            DataSet ds = new DataSet();
            int res = SqlManager.ExecuteSql(SQLQuery, ref ds);
            #endregion

            #region Displaying
            if (res < 1)
                MessageBox.Show("Aucun patient n'est plus vieux que " + nud_Age.Value + " ans!", "Information",
                     MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
            {
                dgv_Age.DataSource = ds.Tables[0];
                for (int i = 0; i < dgv_Age.Columns.Count; i++)
                    dgv_Age.Columns[i].Visible = false;

                dgv_Age.Columns["Nom"].Visible = true;
                dgv_Age.Columns["Prenom"].Visible = true;
                dgv_Age.Columns["DateNaissance"].Visible = true;
                dgv_Age.Columns["DateNaissance"].HeaderText = "Date de naissance";
            }
            #endregion
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la gestionnaire de statistiques
        /// </summary>
        public StatManager StatManager { get { return vStatManager; } set { vStatManager = value; } }
        #endregion
    }
}